#' Scorecards function create simple scorecards by start date
#' 
#'@description function to create scorecards for single system and reference 
#'  combination, showing data by start date.
#'  
#'@param data is an aregay of spatically aggregated metrics data, containing the
#'   following dimensions; system, reference, region, sdate, ftime, metric.
#'@param system.name a vector of character strings defining the names of the 
#'  system names following the archive.yml format from verification suite 
#'@param reference.name a vector of character strings defining the names of 
#'  the references following the archive.yml format from verification suite 
#'  Pending to be test with more than one.
#'@param var.name a character string following the format from 
#'  variable-dictionary.yml from verification suite 
#'@param start.year a numeric indicating the start year of the reference period
#'@param end.year a numeric indicating the end year of the reference period
#'@param start.date a vector of character strings indicating the start months
#'@param forecast.month a vector of numeric indicating the forecast months
#'@param region.names a vector of character strings containing names of the regions
#'@param metrics a vector of character strings containing names of the metrics
#'@param title a character string of the desired output scorecard title
#'  
#'@example 
#'scorecard_plot <- SC_plot_simple(data = aggregated_metrics,
#'                                 system.name = c('ecmwfs5', 'dwds21')  ,
#'                                 reference.name = 'era5',
#'                                 var.name = 'tas',
#'                                 start.year = 1993,
#'                                 end.year = 2016,
#'                                 start.months = 1:12
#'                                 forecast.months = 1:6,
#'                                 region.names = c('Global', 'Europe'),
#'                                 metrics = c('Mean bias', 'Correlation', 'RPSS', 'CRPSS', 'Spread-to-Error Ratio'),
#'                                 column.title = 'Start date' ## or 'Target month'
#'                                 title = 'scorecard test')


SC_plot_simple <- function(data,
                           system.name,
                           reference.name,
                           variable,
                           start.year,
                           end.year,
                           forecast.months,
                           region.names,
                           metrics,
                           column.title,
                           fileout){

## For testing:
data <- round(aggregated_metrics, 2)
system.name <- c('ecmwfs5', 'dwds21')  
reference.name <-'era5'
var.name <- 'tas'
start.year <- 1993
end.year <- 2016
start.months <- sprintf("%02d", 1:12)
forecast.months <- 1:6
region.names <- c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
metrics <- c('mean_bias', 'enscorr', 'rpss', 'crpss','enssprerr')
#metrics <- 'metric'
column.title <- 'Start date'
fileout <- '/esarchive/scratch/nmilders/scorecards_data/saved_results/'


## Make function independent of previous functions
  ## For use with verification suit and previous functions a wrapper will be developed

  # ## Define parameters of scorecard (for linking with function SC_create_scorecard.R)
  # file_metric <- attributes(data)$metrics
  # start.months <- attributes(data)$start.months
  # regions.names <- names(attributes(data)$regions)
  # forecast.months <- attributes(data)$forecast.months  
  # reference.name <- attributes(data)$reference.name
  # system.name <- attributes(data)$system.name[1] ## selecting only first system for testing

  n.start.months <- as.numeric(dim(data)["sdate"]) ## TO DO: create sdate_dim
  n.ftime.months <- as.numeric(dim(data)["time"]) ## TO DO: create ftime_dim
  n.metrics <- as.numeric(dim(data)["metric"]) ## TO DO: create met_dim
  n.regions <- as.numeric(dim(data)["region"]) ## TO DO: create reg_dim

  ## Define table size
  n.rows <- n.regions * n.ftime.months ##needed?
  n.columns <- 2 + (n.metrics * n.start.months)
  
  ## Define table columns
  column.names <- c("Region", "Forecast month", rep(c(month.abb[as.numeric(start.months)]), n.metrics))
  region.column <- region.names ## Need to fix table column
  ftime.column <- rep(as.character(forecast.months), n.regions) 

  ## Table names
  metric.names <- metrics ## TO DO: get proper names from archive file
  #var.name <- ## paste0(toupper(substring(var.name, 1, 1)), substring(str_to_lower(var.name), 2, nchar(var.name)))
  
  region.names.figure <- rep("", n.regions * n.ftime.months)
  for(reg in 1:n.regions) region.names.figure[floor(n.ftime.months/2) + (reg-1) * n.ftime.months + 1] = region.names[reg]


  ## Loop over system and reference for each scorecard plot
  for (sys in 1:dim(data)['system']){
    for (ref in 1:dim(data)['reference']) {
      
      ## Define scorecard table names
      options(stringsAsFactors = FALSE) 
      table.title <- data.frame(c1=paste0(var.name, " of ", system.name[sys]), c2 = n.columns)
      header.names <- as.data.frame(data.frame(c1=c("", metric.names), c2=c(2, rep(n.start.months, n.metrics)))) ## TO DO: need to include metric units
      header.names2 <- as.data.frame(data.frame(c1=c("", paste0(rep(column.title, n.metrics))), c2=c(2, rep(n.start.months, n.metrics))))
      ref.label <- data.frame(c1=paste0("(Ref: ", reference.name[ref], " ", start.year, "-", end.year, ")"), c2 = n.columns)

      
      ## Define data inside the scorecards table
      for(reg in 1:n.regions){
          table_metric <- data.frame(Forecast_month = as.character(forecast.months))
          for(met in 1:n.metrics){
            table_metric <- data.frame(table_metric, data[sys,ref,met,,,reg])   
          }
          if(reg == 1){
            table_reg <- table_metric
          } else {
            table_reg <- rbind(table_reg, table_metric)
          }
      } ## close loop on r
      
      ## All data for plotting in table
      table <- data.frame(region = region.names.figure, table_reg)
      

      ## Define breaks for each metric based of metric position:
      metric.breaks <- list('mean_bias' = c(-1, -0.8, -0.6, -0.4, -0.2, 0, 0.2, 0.4, 0.6, 0.8, 1),
                            'enscorr' = c(-1, -0.8, -0.6, -0.4, -0.2, 0, 0.2, 0.4, 0.6, 0.8, 1),
                            'rpss' = c(-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5),
                            'crpss' = c(-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5),
                            #'crpss' = c(-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5),
                            'enssprerr' = c(0, 0.6, 0.7, 0.8, 0.9, 1, 1.2, 1.4, 1.6, 1.8, 2))
      
    
      ## Define table colors
      palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')
      colorunder <- "#04040E"
      colorsup <- "#730C04"
      
      metric.col <- array(colorunder, c(n.ftime.months * n.regions, n.columns-2)) ## gray is to detect outliers
      metric.text.col <- array("#2A2A2A", c(n.ftime.months * n.regions, n.columns-2)) ## black is the default text color
      metric.text.bold <- array(FALSE, c(n.ftime.months * n.regions, n.columns-2)) ## TRUE is the default bold value

      table_temp <- array(unlist(table[3:n.columns]), dim=c(n.rows,n.columns-2)) ##as.matrix(table[3:n.columns])

      for(met in 1:n.metrics){
        metric.int <- metric.breaks[[metrics[met]]]
        for(rr in 1:n.rows){
         for(stm in 1:n.start.months){
          for(pp in 1:(length(metric.int)-1)){
            if(is.nan(table_temp[rr,((met - 1) * n.start.months + stm)])){
              metric.col[rr,((met - 1) * n.start.months + stm)] <- "gray"
            } else {
              if(table_temp[rr,((met - 1) * n.start.months + stm)] >= metric.int[pp] && table_temp[rr,((met - 1) * n.start.months + stm)] <= metric.int[pp+1])
                metric.col[rr,((met - 1) * n.start.months + stm)] <- palette[pp]
              
              if(table_temp[rr,((met - 1) * n.start.months + stm)] < metric.int[1])
                metric.col[rr,((met - 1) * n.start.months + stm)] <- colorunder
              if(table_temp[rr,((met - 1) * n.start.months + stm)] >= metric.int[length(metric.int)])
                metric.col[rr,((met - 1) * n.start.months + stm)] <- colorsup
            }
            
              ## color text in white and bold if background is white or dark blue or dark red:
              if(is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) ||
                 (!is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) && pp == 1 && table_temp[rr,((met - 1) * n.start.months + stm)] < metric.int[2]) ||
                 (!is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) && pp == 2 && table_temp[rr,((met - 1) * n.start.months + stm)] < metric.int[3]) ||
                 (!is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) && pp == (length(metric.int) - 1) && table_temp[rr,((met - 1) * n.start.months + stm)] >= metric.int[length(metric.int) - 1]) ||
                 (!is.nan(table_temp[rr,((met - 1) * n.start.months + stm)]) && pp == (length(metric.int) - 2) && table_temp[rr,((met - 1) * n.start.months + stm)] >= metric.int[length(metric.int) - 2]))
              {
                metric.text.col[rr,((met - 1) * n.start.months + stm)] <- "white"
                metric.text.bold[rr,((met - 1) * n.start.months + stm)] <- TRUE
              }
              
            } ## close loop on pp
          } ## close loop on stm
        } ## close loop on rr
      } ## close loop on met


      metric.col.df <- data.frame(metric.col)
      metric.text.col.df <- data.frame(metric.text.col)
      metric.text.bold.df <- data.frame(metric.text.bold)
      
 
      ## Create HTML table
      table.html.part <- list()
      table.html.part[[1]] <- kbl(table, escape = F, col.names = column.names, align = rep("c", n.columns)) %>%
        kable_paper("hover", full_width = F) %>%
        add_header_above(header = header.names2, font = 14)%>%
        add_header_above(header = ref.label, align = "left", font_size = 14) %>% 
        add_header_above(header = header.names, font_size = 20) %>%
        add_header_above(header = table.title, font_size = 20, align = "left")
      
      for(i in 1:n.metrics){ 
        if(i == 1) { my.background <- metric.col.df[1:12]; my.text.color <- metric.text.col.df[1:12]; my.bold <- metric.text.bold.df[1:12]}
        if(i == 2) { my.background <- metric.col.df[13:24]; my.text.color <- metric.text.col.df[13:24]; my.bold <- metric.text.bold.df[13:24]}
        if(i == 3) { my.background <- metric.col.df[25:36]; my.text.color <- metric.text.col.df[25:36]; my.bold <- metric.text.bold.df[25:36]}
        if(i == 4) { my.background <- metric.col.df[37:48]; my.text.color <- metric.text.col.df[37:48]; my.bold <- metric.text.bold.df[37:48]}
        if(i == 5) { my.background <- metric.col.df[49:60]; my.text.color <- metric.text.col.df[49:60]; my.bold <- metric.text.bold.df[49:60]}

        for(stm in 1:n.start.months){ ## loop on start months  
          # my.background <- metric.col.df[(i - 1) * n.start.months + stm] 
          # my.text.color <- metric.text.col.df[(i - 1) * n.start.months + stm ] 
          # my.bold <- metric.text.bold.df[(i - 1) * n.start.months + stm]
        
          table.html.part[[(i - 1) * n.start.months + stm + 1]] <-
            column_spec(table.html.part[[(i - 1) * n.start.months + stm]], 2 + n.start.months * (i - 1) + stm, background = my.background[,stm], color = my.text.color[,stm], bold = my.bold[,stm])
        
          } ## close loop on start months
      } ## close loop on metrics
      
        
        # for(met in 1:n.metrics){ ## score
        #    for(stm in 1:n.start.months){
        #      #my.background <- metric.col.df[((met - 1) * n.start.months + stm)]; my.text.color <- metric.text.col.df[((met - 1) * n.start.months + stm)]; my.bold <- metric.text.bold.df[((met - 1) * n.start.months + stm)]
        #     column_spec(table.html.part[[(met - 1) * n.start.months + stm]], 
        #                 2 + n.start.months * (met - 1) + stm, 
        #                 background = metric.col.df[,(met - 1) * n.start.months +stm], 
        #                 color = metric.text.col.df[,(met - 1) * n.start.months +stm], 
        #                 bold = metric.text.bold.df[,(met - 1) * n.start.months +stm])
        #     #column_spec(table.html.part[[(met - 1) * n.start.months + stm]], 2 + n.start.months * (met - 1) + stm, background = my.background[,stm], color = my.text.color[,stm], bold = my.bold[,stm])
        #     
        #     }
        # }

      n.last.list <- n.metrics * n.start.months +1
       
      table.html <- column_spec(table.html.part[[n.last.list]], 1, bold = TRUE, width_min = "4cm", width_max = "4cm") %>%
        column_spec(1, bold = TRUE, width_min = "4cm", width_max = "4cm") %>%
        column_spec(2, bold = TRUE, width_max = "1cm") %>%
        #column_spec(3:n.columns, width_max = "0.9cm", width_min = "0.9cm") %>% ## numbers appear too big
        column_spec(c(1, 2, 2+n.start.months, 2+n.start.months*2, 2+n.start.months*3, 2+n.start.months*4), border_right = "2px solid black") %>% ## Need to make flexible for number of metrics
        ##column_spec(c(1,2,14,26,38,50), border_right = "2px solid black") %>%
      #### use function cumsum !! 
        column_spec(1, border_left = "4px solid gray") %>%
        column_spec(n.columns, border_right = "4px solid gray") %>%
        row_spec(seq(from = 0, to = n.ftime.months * n.regions, by = n.ftime.months), extra_css = "border-bottom: 2px solid black", hline_after = TRUE) #1px solid")
      

      ## Create color bar legends
      if('rpss' %in% metrics | 'crpss' %in% metrics){
        png(filename= paste0(fileout, "/scorecard_legend.png"), width=455, height=40)
        ColorBar(brks = metric.breaks[['rpss']], cols = palette, vertical = FALSE, var_limits=c(-10000000, 1), col_inf = colorunder, col_sup = colorsup) # "#000080")
        dev.off()
      }
      
      if('mean_bias' %in% metrics){
        png(filename= paste0(fileout, "/scorecard_legend_bias.png"), width=455, height=40)
        ColorBar(brks = metric.breaks[['mean_bias']], cols = palette, vertical = FALSE, triangle_ends_scale = 1, col_inf = colorunder, col_sup = colorsup) 
        dev.off()
      }
      
      if('enscorr' %in% metrics){
        png(filename= paste0(fileout, "/scorecard_legend_corr.png"), width=455, height=40)
        ColorBar(brks = metric.breaks[['enscorr']], cols = palette, vertical = FALSE, triangle_ends_scale = 0, col_inf = colorunder)
        dev.off()
      }
      
      if('enssprerr' %in% metrics){
        png(filename= paste0(fileout, "/scorecard_legend_enssprerr.png"), width=455, height=40)
        ColorBar(brks = metric.breaks[['enssprerr']], cols = palette, vertical = FALSE, triangle_ends_scale = 1,  col_sup = colorsup)
        dev.off()
      }
      
      currentworkingdir <- getwd()
      setwd(fileout)
      
      ## Add white space to the left of the first color bar legend
      system(paste0("convert ./scorecard_legend_bias.png -background white -splice 225x0 ./scorecard_legend_bias_full.png"))
      
      ## Paste all legends together
      system(paste0("montage ./scorecard_legend_bias_full.png ./scorecard_legend_corr.png ./scorecard_legend.png ./scorecard_legend.png ./scorecard_legend_enssprerr.png -tile 5x1 -geometry -4+10 ./legend_scorecard.png"))
      unlink('scorecard_legend*')
      setwd(currentworkingdir)
      
      ## Save html table as png
      my.file <- paste0(fileout, '/scorecard.png')
      try(save_kable(table.html, file = my.file), silent = TRUE)
      
      
      ## Remove temporal directories (not automatically removed due to error with save_kable)
      unlink(paste0(fileout, './saved_results/lib*'), recursive = TRUE) # clean temporal directories (not automatically removed because of error with save_kable)
      
      ## Add white space to the top of the scorecards, to be cut when adding legend:
      setwd(fileout)
      system(paste0("convert ./scorecard.png -background white -splice 0x20 ./scorecard.png"))
      
      ## Add the legends below the scorecard table
      system(paste0("montage ./scorecard.png ./legend_scorecard.png -tile 1x2 -geometry +0-10 ./scorecard.png"))
      
      
      ## Save scorecard png in desired directory
      dirout <- initialfunction_save_scorecard(system.name, reference.name, var.filename, start.year, end.year, "type-1")
      system(paste0("mv ", './scorecard.png', paste0(" ", dirout)))
      return(print(paste0("Scorecard type 1 figure saved")))

    } ## close if on reference
  } ## close if on system
  
}